#!/usr/bin/env python3

import json
import os
import subprocess
import sys
import threading
import time
import re

def check_and_import(modules):
    missing_modules = []
    for module_name in modules:
        try:
            globals()[module_name] = __import__(module_name)
        except ImportError:
            missing_modules.append(module_name)

    if missing_modules:
        for missing_module in missing_modules:
            print(f"Ошибка: Модуль '{missing_module}' не найден. Установите его с помощью 'pip install {missing_module}'.")
        sys.exit(1)

required_modules = ["paho.mqtt.client", "pyinotify"]
check_and_import(required_modules)

from lib.sys_utils import IqLogger
from lib.cm_monitor import start_monitoring
from lib.wifi_manager import wifi_scan_task
from lib.network_info import connection_info_task
from lib.network_monitor import network_monitor_task
from lib.modem_manager import modem_info_task, collect_initial_sim_info
from lib.mqtt_client import MQTTClient
from lib.cached_network_status import CachedNetworkStatus

if __name__ == "__main__":
    # Финт на случай перезапуска сервиса в процессе работы контроллера
    # Подрузамевается, что SIM карты не будут менять на горячую,
    # значит не нужно перечитывать IMSI SIM карт.
    # После получения IMSI SIM карт, сохраняем в /tmp/ директорию.
    cache_restored = CachedNetworkStatus().restore()
    mqtt_client = MQTTClient(logger=IqLogger())

    wifi_scan_thread = threading.Thread(target=wifi_scan_task, args=(300, ))
    wifi_scan_thread.daemon = True
    wifi_scan_thread.start()

    wifi_scan_thread = threading.Thread(target=connection_info_task, args=(120, ))
    wifi_scan_thread.daemon = True
    wifi_scan_thread.start()

    file_monitor_thread = threading.Thread(target=start_monitoring)
    file_monitor_thread.daemon = True
    file_monitor_thread.start()

    # Первичное заполнение информации по SIM только при включении контроллера
    # т.к. кеш храним в /tmp/
    if not cache_restored:
        time.sleep(10) # ждем обработки настроек сети
        collect_initial_sim_info()
        CachedNetworkStatus().save()

    modem_info_thread = threading.Thread(target=modem_info_task, args=(30, ))
    modem_info_thread.daemon = False
    modem_info_thread.start()

    network_monitor_thread = threading.Thread(target=network_monitor_task)
    network_monitor_thread.daemon = False
    network_monitor_thread.start()
